'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
var isFunction = exports.isFunction = function isFunction(func) {
  return typeof func === 'function';
};

var isString = exports.isString = function isString(text) {
  return typeof text === 'string' || text instanceof String;
};

var isObject = exports.isObject = function isObject(object) {
  return !isFunction(object) && object === Object(object);
};

var flattenObjectComponents = function flattenObjectComponents(components) {
  return components.reduce(function (collector, component) {
    if (isObject(component)) {
      for (var key in component) {
        if (component.hasOwnProperty(key)) collector.push(component[key]);
      }
    } else {
      collector.push(component);
    }
    return collector;
  }, []);
};

var getDependencies = exports.getDependencies = function getDependencies(components, type, attrs) {
  var toResolve = flattenObjectComponents(components).filter(function (component) {
    return component && component.asyncResolve && component.asyncResolve[type];
  }).map(function (component) {
    return component.asyncResolve[type];
  }).filter(function (resolve) {
    return resolve;
  });
  var promises = [].concat.apply([], toResolve).map(function (resolve) {
    return Promise.resolve(resolve(attrs));
  });
  return Promise.all(promises);
};